from shop_system_models.consts.messages.admin import (
    ADMIN_ORDER_CREATED_MESSAGE,
    ADMIN_ORDER_UPDATED_STATUS_MESSAGE,
)
from shop_system_models.shop_api.bot.request import MessagesType

ADMIN_MESSAGES = {
    MessagesType.admin_order_created: ADMIN_ORDER_CREATED_MESSAGE,
    MessagesType.admin_order_updated: ADMIN_ORDER_UPDATED_STATUS_MESSAGE,
}

PAYMENT_MESSAGE = {"RU": "Способ оплаты: ", "EN": "Payment method: "}

OPEN_TG_PAYMENT_DETAILS_MESSAGE = {
    "RU": "Оплата заказа производится по банковской карте с помощью платежного провайдера телеграмма.",
    "EN": "Order payment is made by bank card using the telegram payment provider.",
}

OPEN_TG_STARS_DETAILS_MESSAGE = {
    "RU": "Оплата заказа производится с использованием Telegram Stars",
    "EN": "Order payment is made using Telegram Stars",
}



from typing import Optional

from pydantic import BaseModel

from shop_system_models.shop_api.bot.response import OrderStatusResponseModel
from shop_system_models.shop_api.links import LinkBlocksModel
from shop_system_models.shop_api.shop.response.orders import DeliveryTypeResponseModel
from shop_system_models.shop_api.shop.response.payment_methods import PaymentMethodDBResponseModel
from shop_system_models.shop_api.shop.response.users import UserResponseModel


class ContentDataResponseModel(BaseModel):
    blocks: LinkBlocksModel | None
    delivery_types: list[DeliveryTypeResponseModel] | None
    payment_methods: list[PaymentMethodDBResponseModel] | None


class InitializationDataResponseModel(ContentDataResponseModel):
    web_app_url: str
    user: UserResponseModel
    search_enabled: Optional[bool] = False
    orders_statuses: list[OrderStatusResponseModel] | None



from typing import Annotated

from annotated_types import Ge
from pydantic import BaseModel
from typing_extensions import Doc


class DBProductInBasket(BaseModel):
    id: str  # ID продукта
    count: int  # его кол-во


class BasketModel(BaseModel):
    user_id: str | None = None  # Not uses in request
    order_id: str | None = None  # Not uses in request
    products_id: list[DBProductInBasket] = []
    amount: float | None = 0  # Not uses in request
    preview_url: str = ""


class DBProductInBasketV2(BaseModel):
    id: str  # product ID
    unique_id: str  # unique product ID in the basket, used it for product deleting
    extra_option_ids: list[str] = []


class BasketModelV2(BaseModel):
    user_id: str | None = None
    order_id: str | None = None
    products_id: list[DBProductInBasketV2] = []
    coupon: str | None = None
    coupon_discount: Annotated[
        float, Doc("The amount of discount from attached coupon, if any.")
    ] = 0
    amount: Annotated[float, Ge(0)] = 0  # this amount already includes discount
    preview_url: str = ""


class CouponInputData(BaseModel):
    code: str



"""Shop request models for API communication."""

from shop_system_models.shop_api.shop.request.orders import (
    AddressModel,
    BookingOrderModel,
    Coordinates,
    CreateOrderModel,
    DeliveryTypeModel,
    ExtraField,
    ExtraFieldPayload,
    OrderDeliveryTypeModel,
    OrderModel,
)



# import re


from pydantic import BaseModel


class UserHeadersUpdatingModel(BaseModel):
    tg_id: int
    first_name: str | None = ""
    last_name: str | None = ""
    tg_language: str | None = ""
    username: str | None = None


class UserModel(UserHeadersUpdatingModel):
    password: str | None = None
    preview_url: str | None = None
    rights: str = "user"
    delivery_addresses: list[str] | None = None


class ReferralUser(BaseModel):
    tg_id: int
    first_name: str
    last_name: str | None = None
    username: str | None = None


class InvitedUser(ReferralUser):
    tg_language: str | None = ""
    language: str | None = None


class UserDeploymentModel(BaseModel):
    id: str
    first_name: str
    last_name: str | None = None
    username: str | None = None
    tg_id: int
    tg_language: str = ""
    invited_by_id: str | None = None
    invited_by_user: ReferralUser | None = None
    roles: list[str] | None = None
    shops_available: int = 5
    invited_users: list[InvitedUser] | None



from typing import Optional

from pydantic import BaseModel


class DeployVariables(BaseModel):
    shop_name: str
    shop_id: str
    shop_template_id: str
    orders_chat_id: int
    contact_email: str
    tg_admin_id: int
    contact_phone: Optional[str]
    bot_token: Optional[str]
    invite_user: bool = True



from pydantic import BaseModel

from shop_system_models.shop_api.bot.request import OrderStatusModel


class OrderStatusResponseModel(OrderStatusModel):
    id: str


class OrderStatusResponseModelWithShopName(BaseModel):
    shop_name: str
    order_statuses: list[OrderStatusResponseModel]



from pydantic import field_validator

from shop_system_models.shop_api.shop.request.payment_methods import PaymentMethodModel


class PaymentMethodDBResponseModel(PaymentMethodModel):
    id: str


class PaymentMethodResponseModel(PaymentMethodDBResponseModel):
    @field_validator("payment_data", mode="before")
    def back_zeros(cls, value):
        return "********"



ISO3166 = {
    "AF": "AFGHANISTAN",
    "AX": "ÅLAND ISLANDS",
    "AL": "ALBANIA",
    "DZ": "ALGERIA",
    "AS": "AMERICAN SAMOA",
    "AD": "ANDORRA",
    "AO": "ANGOLA",
    "AI": "ANGUILLA",
    "AQ": "ANTARCTICA",
    "AG": "ANTIGUA AND BARBUDA",
    "AR": "ARGENTINA",
    "AM": "ARMENIA",
    "AW": "ARUBA",
    "AU": "AUSTRALIA",
    "AT": "AUSTRIA",
    "AZ": "AZERBAIJAN",
    "BS": "BAHAMAS",
    "BH": "BAHRAIN",
    "BD": "BANGLADESH",
    "BB": "BARBADOS",
    "BY": "BELARUS",
    "BE": "BELGIUM",
    "BZ": "BELIZE",
    "BJ": "BENIN",
    "BM": "BERMUDA",
    "BT": "BHUTAN",
    "BO": "BOLIVIA, PLURINATIONAL STATE OF",
    "BQ": "BONAIRE, SINT EUSTATIUS AND SABA",
    "BA": "BOSNIA AND HERZEGOVINA",
    "BW": "BOTSWANA",
    "BV": "BOUVET ISLAND",
    "BR": "BRAZIL",
    "IO": "BRITISH INDIAN OCEAN TERRITORY",
    "BN": "BRUNEI DARUSSALAM",
    "BG": "BULGARIA",
    "BF": "BURKINA FASO",
    "BI": "BURUNDI",
    "KH": "CAMBODIA",
    "CM": "CAMEROON",
    "CA": "CANADA",
    "CV": "CAPE VERDE",
    "KY": "CAYMAN ISLANDS",
    "CF": "CENTRAL AFRICAN REPUBLIC",
    "TD": "CHAD",
    "CL": "CHILE",
    "CN": "CHINA",
    "CX": "CHRISTMAS ISLAND",
    "CC": "COCOS (KEELING) ISLANDS",
    "CO": "COLOMBIA",
    "KM": "COMOROS",
    "CG": "CONGO",
    "CD": "CONGO, THE DEMOCRATIC REPUBLIC OF THE",
    "CK": "COOK ISLANDS",
    "CR": "COSTA RICA",
    "CI": "CÔTE D'IVOIRE",
    "HR": "CROATIA",
    "CU": "CUBA",
    "CW": "CURAÇAO",
    "CY": "CYPRUS",
    "CZ": "CZECH REPUBLIC",
    "DK": "DENMARK",
    "DJ": "DJIBOUTI",
    "DM": "DOMINICA",
    "DO": "DOMINICAN REPUBLIC",
    "EC": "ECUADOR",
    "EG": "EGYPT",
    "SV": "EL SALVADOR",
    "GQ": "EQUATORIAL GUINEA",
    "ER": "ERITREA",
    "EE": "ESTONIA",
    "ET": "ETHIOPIA",
    "FK": "FALKLAND ISLANDS (MALVINAS)",
    "FO": "FAROE ISLANDS",
    "FJ": "FIJI",
    "FI": "FINLAND",
    "FR": "FRANCE",
    "GF": "FRENCH GUIANA",
    "PF": "FRENCH POLYNESIA",
    "TF": "FRENCH SOUTHERN TERRITORIES",
    "GA": "GABON",
    "GM": "GAMBIA",
    "GE": "GEORGIA",
    "DE": "GERMANY",
    "GH": "GHANA",
    "GI": "GIBRALTAR",
    "GR": "GREECE",
    "GL": "GREENLAND",
    "GD": "GRENADA",
    "GP": "GUADELOUPE",
    "GU": "GUAM",
    "GT": "GUATEMALA",
    "GG": "GUERNSEY",
    "GN": "GUINEA",
    "GW": "GUINEA-BISSAU",
    "GY": "GUYANA",
    "HT": "HAITI",
    "HM": "HEARD ISLAND AND MCDONALD ISLANDS",
    "VA": "HOLY SEE (VATICAN CITY STATE)",
    "HN": "HONDURAS",
    "HK": "HONG KONG",
    "HU": "HUNGARY",
    "IS": "ICELAND",
    "IN": "INDIA",
    "ID": "INDONESIA",
    "IR": "IRAN, ISLAMIC REPUBLIC OF",
    "IQ": "IRAQ",
    "IE": "IRELAND",
    "IM": "ISLE OF MAN",
    "IL": "ISRAEL",
    "IT": "ITALY",
    "JM": "JAMAICA",
    "JP": "JAPAN",
    "JE": "JERSEY",
    "JO": "JORDAN",
    "KZ": "KAZAKHSTAN",
    "KE": "KENYA",
    "KI": "KIRIBATI",
    "KP": "KOREA, DEMOCRATIC PEOPLE'S REPUBLIC OF",
    "KR": "KOREA, REPUBLIC OF",
    "KW": "KUWAIT",
    "KG": "KYRGYZSTAN",
    "LA": "LAO PEOPLE'S DEMOCRATIC REPUBLIC",
    "LV": "LATVIA",
    "LB": "LEBANON",
    "LS": "LESOTHO",
    "LR": "LIBERIA",
    "LY": "LIBYAN ARAB JAMAHIRIYA",
    "LI": "LIECHTENSTEIN",
    "LT": "LITHUANIA",
    "LU": "LUXEMBOURG",
    "MO": "MACAO",
    "MK": "MACEDONIA, THE FORMER YUGOSLAV REPUBLIC OF",
    "MG": "MADAGASCAR",
    "MW": "MALAWI",
    "MY": "MALAYSIA",
    "MV": "MALDIVES",
    "ML": "MALI",
    "MT": "MALTA",
    "MH": "MARSHALL ISLANDS",
    "MQ": "MARTINIQUE",
    "MR": "MAURITANIA",
    "MU": "MAURITIUS",
    "YT": "MAYOTTE",
    "MX": "MEXICO",
    "FM": "MICRONESIA, FEDERATED STATES OF",
    "MD": "MOLDOVA, REPUBLIC OF",
    "MC": "MONACO",
    "MN": "MONGOLIA",
    "ME": "MONTENEGRO",
    "MS": "MONTSERRAT",
    "MA": "MOROCCO",
    "MZ": "MOZAMBIQUE",
    "MM": "MYANMAR",
    "NA": "NAMIBIA",
    "NR": "NAURU",
    "NP": "NEPAL",
    "NL": "NETHERLANDS",
    "NC": "NEW CALEDONIA",
    "NZ": "NEW ZEALAND",
    "NI": "NICARAGUA",
    "NE": "NIGER",
    "NG": "NIGERIA",
    "NU": "NIUE",
    "NF": "NORFOLK ISLAND",
    "MP": "NORTHERN MARIANA ISLANDS",
    "NO": "NORWAY",
    "OM": "OMAN",
    "PK": "PAKISTAN",
    "PW": "PALAU",
    "PS": "PALESTINIAN TERRITORY, OCCUPIED",
    "PA": "PANAMA",
    "PG": "PAPUA NEW GUINEA",
    "PY": "PARAGUAY",
    "PE": "PERU",
    "PH": "PHILIPPINES",
    "PN": "PITCAIRN",
    "PL": "POLAND",
    "PT": "PORTUGAL",
    "PR": "PUERTO RICO",
    "QA": "QATAR",
    "RE": "RÉUNION",
    "RO": "ROMANIA",
    "RU": "RUSSIAN FEDERATION",
    "RW": "RWANDA",
    "BL": "SAINT BARTHÉLEMY",
    "SH": "SAINT HELENA, ASCENSION AND TRISTAN DA CUNHA",
    "KN": "SAINT KITTS AND NEVIS",
    "LC": "SAINT LUCIA",
    "MF": "SAINT MARTIN (FRENCH PART)",
    "PM": "SAINT PIERRE AND MIQUELON",
    "VC": "SAINT VINCENT AND THE GRENADINES",
    "WS": "SAMOA",
    "SM": "SAN MARINO",
    "ST": "SAO TOME AND PRINCIPE",
    "SA": "SAUDI ARABIA",
    "SN": "SENEGAL",
    "RS": "SERBIA",
    "SC": "SEYCHELLES",
    "SL": "SIERRA LEONE",
    "SG": "SINGAPORE",
    "SX": "SINT MAARTEN (DUTCH PART)",
    "SK": "SLOVAKIA",
    "SI": "SLOVENIA",
    "SB": "SOLOMON ISLANDS",
    "SO": "SOMALIA",
    "ZA": "SOUTH AFRICA",
    "GS": "SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS",
    "SS": "SOUTH SUDAN",
    "ES": "SPAIN",
    "LK": "SRI LANKA",
    "SD": "SUDAN",
    "SR": "SURINAME",
    "SJ": "SVALBARD AND JAN MAYEN",
    "SZ": "SWAZILAND",
    "SE": "SWEDEN",
    "CH": "SWITZERLAND",
    "SY": "SYRIAN ARAB REPUBLIC",
    "TW": "TAIWAN, PROVINCE OF CHINA",
    "TJ": "TAJIKISTAN",
    "TZ": "TANZANIA, UNITED REPUBLIC OF",
    "TH": "THAILAND",
    "TL": "TIMOR-LESTE",
    "TG": "TOGO",
    "TK": "TOKELAU",
    "TO": "TONGA",
    "TT": "TRINIDAD AND TOBAGO",
    "TN": "TUNISIA",
    "TR": "TURKEY",
    "TM": "TURKMENISTAN",
    "TC": "TURKS AND CAICOS ISLANDS",
    "TV": "TUVALU",
    "UG": "UGANDA",
    "UA": "UKRAINE",
    "AE": "UNITED ARAB EMIRATES",
    "GB": "UNITED KINGDOM",
    "US": "UNITED STATES",
    "UM": "UNITED STATES MINOR OUTLYING ISLANDS",
    "UY": "URUGUAY",
    "UZ": "UZBEKISTAN",
    "VU": "VANUATU",
    "VE": "VENEZUELA, BOLIVARIAN REPUBLIC OF",
    "VN": "VIET NAM",
    "VG": "VIRGIN ISLANDS, BRITISH",
    "VI": "VIRGIN ISLANDS, U.S.",
    "WF": "WALLIS AND FUTUNA",
    "EH": "WESTERN SAHARA",
    "YE": "YEMEN",
    "ZM": "ZAMBIA",
    "ZW": "ZIMBABWE",
}



from typing import Optional

from shop_system_models.deployment_api.users import UserModel


class UserResponseModel(UserModel):
    id: str
    is_service: bool = False
    is_blocked: bool = False
    message_thread_id: Optional[int] = None



from enum import Enum

from pydantic.types import Enum


class OrderProcesses(str, Enum):
    without_payment = "The-devs.Shop.CreateOrder"
    tg_payment = "The-devs.Shop.MakeOrderWithTelegramPayments"
    ton_payment = "The-devs.Shop.MakeOrderWithTONPayments"
    life_pay = "The-devs.Shop.CreateLinkPaymentsOrder"


class PaymentTypes(str, Enum):
    manual_payment_request = "ManualPaymentRequest"
    external_card_payment_provider = "ExternalCardPaymentProvider"
    crypto_ton = "CryptoTON"
    xtr = "XTR"
    life_pay = "LifePay"
    yookassa = "yookassa"
    tkassa = "tkassa"
    cloud_payments = "CloudPayments"


class ShopStatuses(str, Enum):
    deploying = "deploying"
    active = "active"
    importing = "importing"
    blocked = "blocked"
    unavailable = "unavailable"
    deleted = "deleted"


class TaskStatuses(str, Enum):
    pending = "pending"
    expired = "expired"
    completed = "completed"


class ServiceNames(str, Enum):
    bot = "bot"


class UserRoles(str, Enum):
    user = "user"
    admin = "admin"
    shop_admin = "shop_admin"
    shop_manager = "shop_manager"


PAYMENTS_PROCESSES_MAPPING = {
    PaymentTypes.manual_payment_request.value: OrderProcesses.without_payment.value,
    PaymentTypes.external_card_payment_provider.value: OrderProcesses.tg_payment.value,
    PaymentTypes.xtr.value: OrderProcesses.tg_payment.value,
    PaymentTypes.crypto_ton.value: OrderProcesses.ton_payment.value,
    PaymentTypes.life_pay.value: OrderProcesses.life_pay.value,
    PaymentTypes.yookassa.value: OrderProcesses.life_pay.value,
    PaymentTypes.tkassa.value: OrderProcesses.life_pay.value,
}



"""Shop response models for API communication."""

from shop_system_models.shop_api.shop.response.orders import (
    DeliveryTypeResponseModel,
    OrderListResponseModel,
    OrderResponseModel,
)
from shop_system_models.shop_api.shop.response.products import (
    PaginationResponseModel,
    PartialPaginationResponseModel,
)



from typing import Optional

from pydantic import BaseModel


class NocoDBConfig(BaseModel):
    nocodb_project_id: Optional[str] = None
    nocodb_categories_table: Optional[str] = None
    nocodb_products_table: Optional[str] = None
    nocodb_orders_table: Optional[str] = None
    nocodb_status_table: Optional[str] = None
    nocodb_bot_commands_table: Optional[str] = None
    nocodb_options_table: Optional[str] = ""
    nocodb_options_category_table: Optional[str] = ""



from pydantic import BaseModel, field_validator

from shop_system_models.shop_api.bot.request import ButtonModel


class BotCommandModel(BaseModel):
    name: str
    message: str

    @field_validator("name", mode="before")
    def command_validation(cls, value):
        if value.startswith("/"):
            return value
        raise ValueError('Bot command should start with "/"')


class NotificationPayloadModel(BaseModel):
    chat_id: int
    message_id: int | None = -1
    text: str
    buttons: list[ButtonModel] = []



from datetime import datetime
from typing import List

from pydantic import BaseModel, field_validator
from pydantic_core.core_schema import ValidationInfo

from shop_system_models.shop_api import MetaBaseModel
from shop_system_models.shop_api.shared import ImageMetadata


class ExtraAttribute(BaseModel):
    name: str
    description: str


class SubmitProductModel(MetaBaseModel):
    name: str
    description: str = ""
    price: float
    final_price: float = 0
    currency: str
    preview_url: list[str] = []
    stock_qty: int
    orders_qty: int = 0
    extra_attributes: list[ExtraAttribute] = []


class SubmitProductCategoriesModel(MetaBaseModel):
    category: list[int]


class ProductModel(SubmitProductModel):
    created: datetime = datetime.fromtimestamp(0)
    updated: datetime = datetime.fromtimestamp(0)
    images: List[ImageMetadata] = []

    @field_validator("final_price", mode="before")
    def set_final_price(cls, value, values: ValidationInfo):
        if not value:
            return values.data.get("price")
        return value



from pydantic import BaseModel

from shop_system_models.shop_api.shop.request.categories import CategoryModel
from shop_system_models.shop_api.shop.response.products import (
    PaginationResponseModel,
    ProductListResponseModel,
    ProductResponseModel,
)


class CategoryResponseModel(CategoryModel):
    id: int


class SubcatsProdsResponseModel(BaseModel):
    subcategories: list[CategoryResponseModel]
    products: list[ProductResponseModel]


class MainCatIncredProdResponseModel(BaseModel):
    categories: list[CategoryResponseModel]
    products: list[ProductResponseModel]


class CategoryListResponseModel(BaseModel):
    categories: list[CategoryResponseModel]
    page_info: PaginationResponseModel


class CategoriesProductsResponseModel(BaseModel):
    categories: CategoryListResponseModel
    products: ProductListResponseModel



from datetime import datetime
from pydantic import BaseModel

class NocoDBConfig(BaseModel):
    nocodb_project_id: str
    nocodb_categories_table: str
    nocodb_products_table: str
    nocodb_orders_table: str
    nocodb_status_table: str
    nocodb_bot_commands_table: str
    nocodb_options_table: str | None
    nocodb_options_category_table: str | None
    agents_definition_table: str | None

class OpeningHours(BaseModel):
    monday: str | None
    tuesday: str | None
    wednesday: str | None
    thursday: str | None
    friday: str | None
    saturday: str | None
    sunday: str | None

class ShopDetailsModel(BaseModel):
    shop_id: str
    shop_name: str
    friendly_name: str
    shop_language: str
    shop_api_url: str
    contact_phone: str | None
    contact_email: str
    orders_chat_id: int
    orders_history_chat_id: int | None
    search_enabled: bool | None
    warehouse_accounting: bool
    bot_url: str | None
    bot_token: str
    payment_token: str | None
    placeholder: str
    order_process: str
    nocodb_config: NocoDBConfig | None
    topic_chat_id: int | None
    premium_expiration_date: datetime | None
    web_app_url: str
    short_description: str | None
    description: str | None



from pydantic import BaseModel


class SubmitReviewModel(BaseModel):
    score: int
    text: str
    images: list[str] | None = []
    user_id: int | None = None  # Анонимный отзыв, если None


class ReviewModel(SubmitReviewModel):
    shop_name: str



import enum
from enum import Enum
from typing import Optional

from pydantic import BaseModel


class OrderStatusTypes(str, enum.Enum):
    CREATED = "CREATED"
    CANCELLED = "CANCELLED"
    PAID = "PAID"


class OrderStatusModel(BaseModel):
    name: str
    message: str
    in_timeline: bool | None = False
    type: Optional[OrderStatusTypes] = None

    # @field_validator('name', )
    # def validate_name_len(cls, value):
    #     if len(value) > 16:
    #         return value[:16]
    #     return value


class MessagesType(str, Enum):
    admin_order_created = "admin_order_created"
    admin_order_updated = "admin_order_updated"
    user_order_created = "user_order_created"
    user_order_updated = "user_order_updated"


class MessageModel(BaseModel):
    type: MessagesType
    text: str


class ButtonType(str, Enum):
    inline_url = "inline_url"
    inline_payload = "inline_payload"
    app_url = "app_url"


class ButtonModel(BaseModel):
    linked_status: OrderStatusModel | None = None
    type: ButtonType
    name: str
    payload: str | None = ""



import re
from typing import Any, Dict, Optional

from pydantic import BaseModel, Field, HttpUrl, field_validator

from shop_system_models.shop_api import MetaBaseModel

# Schemes valid in href (http, https, mailto, tel, and other URI schemes)
ALLOWED_LINK_SCHEMES = frozenset({"http", "https", "mailto", "tel"})
SCHEME_PATTERN = re.compile(r"^[a-zA-Z][a-zA-Z0-9+.-]*$")


def _is_valid_href(value: str) -> bool:
    s = value.strip()
    if not s or ":" not in s:
        return False
    scheme, _ = s.split(":", 1)
    return scheme.lower() in ALLOWED_LINK_SCHEMES or bool(SCHEME_PATTERN.match(scheme))


class ServiceLink(MetaBaseModel):
    """Model representing a link to another service."""

    service_name: str
    url: HttpUrl
    resource_id: str
    link_metadata: Optional[Dict[str, Any]] = Field(default_factory=lambda: {})


class ServiceLinkResponse(MetaBaseModel):
    """Response model for service link operations."""

    link_id: str
    service_name: str
    resource_id: str
    success: bool = True
    error: Optional[str] = None


class LinkBlocksModel(BaseModel):
    contacts: str | None = "https://teletype.in/@tg-shops/09hOkoJjizN"  # Teletype link
    info: str | None = "https://teletype.in/@tg-shops/2vZa_0ykcCF"  # Teletype link

    @field_validator("contacts", "info", mode="before")
    def link_validation(cls, value):
        if not value:
            return value
        s = value.strip() if isinstance(value, str) else value
        if not s:
            return value
        return s if _is_valid_href(str(s)) else ""



from typing import Optional

from pydantic import BaseModel


class Thumbnail(BaseModel):
    signedUrl: Optional[str] = None
    path: Optional[str] = None


class Thumbnails(BaseModel):
    tiny: Optional[Thumbnail]
    small: Optional[Thumbnail]
    card_cover: Optional[Thumbnail]


class ImageMetadata(BaseModel):
    id: Optional[str] = None
    url: Optional[str] = None
    title: Optional[str] = None
    mimetype: Optional[str] = None
    thumbnails: Optional[Thumbnails]
    path: Optional[str] = None
    signedUrl: Optional[str] = None



import datetime
from typing import Annotated

from annotated_types import Ge, Le
from pydantic import BaseModel, StringConstraints, model_validator


class SubmitCouponModel(BaseModel):
    exp_date: datetime.datetime | None = None
    amount: Annotated[float, Ge(0)] | None = None
    discount: Annotated[float, Ge(0), Le(100)] | None = None
    available_qty: Annotated[int, Ge(0)]
    code: Annotated[str, StringConstraints(min_length=1, max_length=10)]

    @model_validator(mode="after")
    def amount_xor_discount_validator(self):
        if self.amount is None and self.discount is None:
            raise ValueError("amount or discount must be specified")

        if self.amount is not None and self.discount is not None:
            raise ValueError("only one of amount and discount must be specified")

        return self


class CouponModel(SubmitCouponModel):
    def get_discount_amount(self, basket_amount: float) -> float:
        """
        Calculate the discount amount.

        It is guaranteed that the discount amount will be less than basket amount,
        i.e. final basket amount won't go below zero.
        """

        if self.amount is not None:
            discount_amount = self.amount
        else:
            assert self.discount is not None, "amount and discount are both None"  # for mypy

            # Round down discount_amount with precision of 2 digits after point
            discount_amount = int(self.discount * basket_amount) / 100

        return min(discount_amount, basket_amount - 1)



ADMIN_ORDER_CREATED_MESSAGE: dict[str, str] = {
    "RU": "{user} создал заказ: {order_number}.\n\n"
    "<b>{order_products}</b>\n"
    "Тип доставки: {order_delivery_type}\n"
    "Данные для доставки: <code>{order_delivery_info}</code>\n"
    "Желаемая дата доставки: {order_date_from}\n"
    "Телефон для связи: <code>{user_contact_number}</code>\n"
    "Комментарий: <code>{order_comment}</code>\n\n"
    "Сумма заказа: <b>{order_amount} {currency}</b>{payment_type_text}\n\n"
    "Установите статус👇",
    "EN": "{user} has created an order: {order_number}.\n\n"
    "<b>{order_products}</b>\n"
    "Delivery type: {order_delivery_type}\n"
    "Delivery info: <code>{order_delivery_info}</code>\n"
    "Desired delivery date: {order_date_from}\n"
    "Contact phone number: <code>{user_contact_number}</code>\n"
    "Comment: <code>{order_comment}</code>\n\n"
    "Order amount: <b>{order_amount} {currency}</b>{payment_type_text}\n\n"
    "Set the status👇",
}

ADMIN_ORDER_UPDATED_STATUS_MESSAGE: dict[str, str] = {
    "RU": "Статус заказа {order_number} от {user} изменен.\n\n"
    "<b>{order_products}</b>\n"
    "Тип доставки: {order_delivery_type}\n"
    "Данные для доставки: <code>{order_delivery_info}</code>\n"
    "Желаемая дата доставки: {order_date_from}\n"
    "Телефон для связи: <code>{user_contact_number}</code>\n"
    "Комментарий: <code>{order_comment}</code>\n\n"
    "Сумма заказа: <b>{order_amount} {currency}</b>{payment_type_text}\n"
    "Обновленный статус: 👉 <b>{order_status}</b>",
    "EN": "The status of the order {order_number} from {user} has been changed.\n\n"
    "<b>{order_products}</b>\n"
    "Delivery type: {order_delivery_type}\n"
    "Delivery info: <code>{order_delivery_info}</code>\n"
    "Desired delivery date: {order_date_from}\n"
    "Contact phone number: <code>{user_contact_number}</code>\n"
    "Comment: <code>{order_comment}</code>\n\n"
    "Order amount: <b>{order_amount} {currency}</b>{payment_type_text}\n"
    "Updated status: 👉 <b>{order_status}</b>",
}


DEFAULT_USER_STATUS_MESSAGES: dict[str, dict[str, str]] = {
    "EN": {
        "Not payed": """Order {order_number} created
{order_products}

Delivery type: {order_delivery_type}
Delivery info: {order_delivery_info}
Delivery Date: {order_date_from}
Comment: {order_comment}
Total Amount: {order_amount} 

Status: Not paid""",
        "Booking request": """Booking request {order_number} created
{order_products}

Delivery type: {order_delivery_type}
From date: {order_booking_details_from_date}
Till date: {order_booking_details_till_date}
Comment: {order_comment}

Status: Booking Requested""",
        "Payed": """Order {order_number} updated
{order_products}
Delivery Type: {order_delivery_type}
Delivery Information: {order_delivery_info}
Desired Delivery Date: {order_date_from}
Comment: {order_comment}
Amount: {order_amount} 

Status: Paid""",
        "Preparing": """Order {order_number} updated
{order_products}
Delivery Type: {order_delivery_type}
Delivery Information: {order_delivery_info}
Desired Delivery Date: {order_date_from}
Comment: {order_comment}
Amount: {order_amount} 

Status: Preparing""",
        "On the way": """Order {order_number} updated
{order_products}
Delivery Type: {order_delivery_type}
Delivery Information: {order_delivery_info}
Desired Delivery Date: {order_date_from}
Comment: {order_comment}
Amount: {order_amount} 

Status: On the way""",
        "Delivered": """Order {order_number} updated
{order_products}
Delivery Type: {order_delivery_type}
Delivery Information: {order_delivery_info}
Desired Delivery Date: {order_date_from}
Comment: {order_comment}
Amount: {order_amount} 

Status: Delivered""",
        "Cancelled": """Order {order_number} updated
{order_products}
Delivery Type: {order_delivery_type}
Delivery Information: {order_delivery_info}
Desired Delivery Date: {order_date_from}
Comment: {order_comment}
Amount: {order_amount} 

Status: Cancelled""",
        "default": """Order {order_number} updated
{order_products}
Delivery Type: {order_delivery_type}
Delivery Information: {order_delivery_info}
Desired Delivery Date: {order_date_from}
Comment: {order_comment}
Amount: {order_amount} 

Status: {status}""",
    },
    "RU": {
        "Не оплачен": """Создан заказ {order_number}
{order_products}

Тип доставки: {order_delivery_type}
Данные для доставки: {order_delivery_info}
Желаемая дата доставки: {order_date_from}
Комментарий: {order_comment}
Сумма заказа: {order_amount} 

Статус: Не оплачен""",
        "Запрос Брони": """Бронь {order_number} создана
{order_products}

{order_booking_details_from_date} - {order_booking_details_till_date}
Комментарий: {order_comment}

Статус: Запрошена Бронь""",
        "Оплачен": """Статус заказа {order_number} обновлен
{order_products}

Тип доставки: {order_delivery_type}
Данные для доставки: {order_delivery_info}
Желаемая дата доставки: {order_date_from}
Комментарий: {order_comment}
Сумма заказа: {order_amount} 

Статус: Оплачен""",
        "Собирается": """Статус заказа {order_number} обновлен
{order_products}

Тип доставки: {order_delivery_type}
Данные для доставки: {order_delivery_info}
Желаемая дата доставки: {order_date_from}
Комментарий: {order_comment}
Сумма заказа: {order_amount} 

Статус: Собирается""",
        "В пути": """Статус заказа {order_number} обновлен
{order_products}

Тип доставки: {order_delivery_type}
Данные для доставки: {order_delivery_info}
Желаемая дата доставки: {order_date_from}
Комментарий: {order_comment}
Сумма заказа: {order_amount} 

Статус: В пути""",
        "Доставлен": """Статус заказа {order_number} обновлен
{order_products}

Тип доставки: {order_delivery_type}
Данные для доставки: {order_delivery_info}
Желаемая дата доставки: {order_date_from}
Комментарий: {order_comment}
Сумма заказа: {order_amount} 

Статус: Доставлен""",
        "Отменен": """Статус заказа {order_number} обновлен
{order_products}

Тип доставки: {order_delivery_type}
Данные для доставки: {order_delivery_info}
Желаемая дата доставки: {order_date_from}
Комментарий: {order_comment}
Сумма заказа: {order_amount} 

Статус: Отменен""",
        "default": """Статус заказа {order_number} обновлен
{order_products}

Тип доставки: {order_delivery_type}
Данные для доставки: {order_delivery_info}
Желаемая дата доставки: {order_date_from}
Комментарий: {order_comment}
Сумма заказа: {order_amount} 

Статус: {status}""",
    },
}



from _typeshed import Incomplete
from pydantic.types import Enum

class OrderProcesses(str, Enum):
    without_payment = 'The-devs.Shop.CreateOrder'
    tg_payment = 'The-devs.Shop.MakeOrderWithTelegramPayments'
    ton_payment = 'The-devs.Shop.MakeOrderWithTONPayments'
    life_pay = 'The-devs.Shop.CreateLinkPaymentsOrder'

class PaymentTypes(str, Enum):
    manual_payment_request = 'ManualPaymentRequest'
    external_card_payment_provider = 'ExternalCardPaymentProvider'
    crypto_ton = 'CryptoTON'
    xtr = 'XTR'
    life_pay = 'LifePay'
    yookassa = 'yookassa'
    tkassa = 'tkassa'
    cloud_payments = 'CloudPayments'

class ShopStatuses(str, Enum):
    deploying = 'deploying'
    active = 'active'
    importing = 'importing'
    blocked = 'blocked'
    unavailable = 'unavailable'
    deleted = 'deleted'

class TaskStatuses(str, Enum):
    pending = 'pending'
    expired = 'expired'
    completed = 'completed'

class ServiceNames(str, Enum):
    bot = 'bot'

class UserRoles(str, Enum):
    user = 'user'
    admin = 'admin'
    shop_admin = 'shop_admin'
    shop_manager = 'shop_manager'

PAYMENTS_PROCESSES_MAPPING: Incomplete



from pydantic import BaseModel

from shop_system_models.consts.enums import PaymentTypes


class Metadata(BaseModel):
    key: str
    value: list[str]


class PaymentMethodModel(BaseModel):
    name: str
    type: PaymentTypes
    payment_data: str | None = None  # payment_token, TON address etc....
    meta: list[Metadata] | None = []

    class Config:
        use_enum_values = True



from pydantic import BaseModel


class PageInfo(BaseModel):
    total_rows: int
    page: int
    page_size: int
    is_first_page: bool
    is_last_page: bool


class ListResponseModel(BaseModel):
    page_info: PageInfo



from pydantic import BaseModel
from shops_payment_processing.models.invoice import ( # type: ignore
    InvoiceWithPaymentLinkMessageModel,  # type: ignore
)

from shop_system_models.shop_api.commands.request import NotificationPayloadModel
from shop_system_models.shop_api.shop.request.orders import (
    InvoiceTGMessageModel,
    InvoiceTONMessageModel,
)
from shop_system_models.shop_api.shop.request.orders import (
    InvoiceWithPaymentLinkMessageModel as InvoiceWithPaymentLinkMessageModelLocal,
)


class SetAdminMessageIdProcess(BaseModel):
    admin_message_id: int | None = None


class SetProcessStatus(BaseModel):
    message: str | None = None
    success: bool | None = None


class ProcessModel(SetAdminMessageIdProcess, SetProcessStatus):
    name: str
    process_key: int
    order_id: str


class ProcessVariables(BaseModel):
    order_id: str
    shop_url: str
    shop_id: str
    shop_api_url: str
    shop_bot_url: str
    user_order_info: NotificationPayloadModel
    admin_order_info: NotificationPayloadModel
    payment_info: (
        InvoiceTGMessageModel
        | InvoiceTONMessageModel
        | InvoiceWithPaymentLinkMessageModel
        | InvoiceWithPaymentLinkMessageModelLocal
        | None
    ) = None
    send_to_admin_dm: bool = False
    shop_language: str



import enum

from pydantic import BaseModel


class PremiumPlanTypes(str, enum.Enum):
    month = "1month"
    three_months = "3months"
    six_months = "6months"
    year = "12months"


class RequestPremiumModel(BaseModel):
    shop_id: str
    premium_type: PremiumPlanTypes



from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel, Field

from shop_system_models.shop_api import MetaBaseModel
from shop_system_models.shop_api.shop.response.baskets import UserBasketResponseModelV2
from shop_system_models.shop_api.shop.response.products import ProductResponseModel


class ExtraFieldPayload(MetaBaseModel):
    """Model for extra field data in orders."""

    name: str
    value: str


class OrderDeliveryTypeModel(BaseModel):
    """Model for order delivery details."""

    name: str
    address: Optional[str] = ""
    amount: Optional[float] = 0.0
    extra_fields_payload: Optional[List[ExtraFieldPayload]] = Field(default_factory=lambda: [])


class Coordinates(MetaBaseModel):
    """Geographic coordinates."""

    latitude: Optional[float] = None
    longitude: Optional[float] = None


class AddressModel(MetaBaseModel):
    """Address model with optional coordinates."""

    coordinates: Optional[Coordinates] = None
    address: Optional[str] = None


class ExtraField(MetaBaseModel):
    """Model for defining extra fields."""

    name: str
    description: str
    is_required: Optional[bool] = None


class DeliveryTypeModel(MetaBaseModel):
    """Delivery type configuration model."""

    name: str
    amount: float = 0.0
    is_address_required: Optional[bool] = False
    address_hint: Optional[str] = ""
    extra_fields: List[ExtraField] = Field(default_factory=list)
    is_timepicker_required: bool = False
    details: Optional[str] = None
    country: Optional[str] = None
    country_code: Optional[str] = None  # ISO format, ex.: RU, KZ, BY...
    city: Optional[str] = None
    delivery_location: Optional[Coordinates] = None
    delivery_radius: Optional[int] = None
    delivery_min_hour: Optional[int] = None
    delivery_max_hour: Optional[int] = None
    delivery_minutes_interval: Optional[int] = None
    delivery_min_day: Optional[int] = None
    delivery_max_day: Optional[int] = None


class BookingOrderModel(MetaBaseModel):
    """Details for a booking order."""

    booking_id: str
    from_date: datetime
    till_date: datetime
    item_id: str


class CreateOrderModel(MetaBaseModel):
    """Model for creating a new order."""

    delivery: Optional[OrderDeliveryTypeModel] = None
    delivery_date_from: datetime = Field(default_factory=lambda: datetime.now())
    comment: Optional[str] = None
    user_contact_number: Optional[str] = None
    payment_type_id: Optional[str] = None
    booking_details: Optional[BookingOrderModel] = None
    preview_url: Optional[str] = ""


class ProductsInBasket(ProductResponseModel):
    count_in_basket: int = 1


class OrderPaymentDetails(BaseModel):
    title: str
    description: str
    expires_at: datetime
    link: str


# This depends on UserBasketResponseModelV2 which we'll define in a separate file
# For now, we'll create a placeholder class that can be updated later
class OrderModel(CreateOrderModel):
    """Complete order model."""

    basket: UserBasketResponseModelV2  # This will be implemented in basket models
    user_id: Optional[str] = None
    basket_id: Optional[str] = None
    status: Optional[str] = None
    client_coordinates: Optional[AddressModel] = None
    created: datetime = Field(default_factory=lambda: datetime.now())
    updated: datetime = Field(default_factory=lambda: datetime.now())
    order_number: str = "#0001"
    process_key: Optional[int] = None
    coupon: Optional[str] = None
    admin_message_id: str | None = None
    payment_data: OrderPaymentDetails | None = None


class LabeledPrice(BaseModel):
    label: str
    amount: int


class InvoiceBaseModel(BaseModel):
    chat_id: int
    order_id: str
    order_number: str  # order_number
    payload: str  # <basket_id>_<order_id>_<order_number> from OrderResponseModel --> its subscription key
    amount: float
    currency: str  # fiat
    payment_address: str
    payment_timeout: int | None = None


class InvoiceTGMessageModel(InvoiceBaseModel):
    description: str  # order_products
    provider_data: str | None = None
    prices: list[LabeledPrice]  # label and amount in coins!
    need_name: bool | None = False
    need_phone_number: bool | None = False
    need_email: bool | None = False
    send_phone_number_to_provider: bool | None = False
    send_email_to_provider: bool | None = False
    is_flexible: bool | None = False
    reply_markup: bool | None = None


class InvoiceWithPaymentLinkMessageModel(InvoiceBaseModel):
    payment_link: str


class InvoiceTONMessageModel(InvoiceBaseModel):
    approved_addresses: list[str] = []
    ton_amount: float


class PaidContentMessage(BaseModel):
    message: str



import datetime

from pydantic import BaseModel

from shop_system_models.shop_api.shop.request.coupons import CouponModel
from shop_system_models.shop_api.shop.response.products import PartialPaginationResponseModel


class CouponResponseModel(CouponModel):
    id: str
    created_at: datetime.datetime


class CouponListResponseModel(BaseModel):
    coupons: list[CouponResponseModel]
    page_info: PartialPaginationResponseModel



from typing import List

from pydantic import BaseModel, Field, field_validator

from shop_system_models.shop_api.shared import ImageMetadata


class SubmitParentCategoryModel(BaseModel):
    parent_category: list[int] = Field(default_factory=list)


class SubmitCategoryModel(BaseModel):
    name: str
    preview_url: str = ""

    @field_validator("name", mode="before")
    def name_validation(cls, value):
        if not value:
            raise ValueError("Name cannot be an empty string.")
        return value


class CategoryModel(SubmitCategoryModel, SubmitParentCategoryModel):
    images: List[ImageMetadata] = []



from pydantic import BaseModel

from shop_system_models.shop_api import MetaBaseModel


class ExtraOptionResponseModel(BaseModel):
    id: str
    name: str
    description: str = ""
    price: float = 0
    preview_url: list[str] = []


class ExtraOptionCategoriesResponseModel(MetaBaseModel):
    id: str
    name: str
    description: str = ""
    choice_count: int = 0  # How many options user can select from this category, 0 means no limit
    is_required: bool | None = (
        False  # If true, user must select at least one option from this category
    )
    options: list[ExtraOptionResponseModel] = []



from pydantic import BaseModel

class UserHeadersUpdatingModel(BaseModel):
    tg_id: int
    first_name: str | None
    last_name: str | None
    tg_language: str | None
    username: str | None

class UserModel(UserHeadersUpdatingModel):
    password: str | None
    preview_url: str | None
    rights: str
    delivery_addresses: list[str] | None

class ReferralUser(BaseModel):
    tg_id: int
    first_name: str
    last_name: str | None
    username: str | None

class InvitedUser(ReferralUser):
    tg_language: str | None
    language: str | None

class UserDeploymentModel(BaseModel):
    id: str
    first_name: str
    last_name: str | None
    username: str | None
    tg_id: int
    tg_language: str
    invited_by_id: str | None
    invited_by_user: ReferralUser | None
    roles: list[str] | None
    shops_available: int
    invited_users: list[InvitedUser] | None



from pydantic import BaseModel, Field


class MetaBaseModel(BaseModel):
    """Base model with metadata support for all shop system models."""

    metadata: dict = Field(default_factory=dict, exclude=True)



from pydantic import BaseModel

from shop_system_models.shop_api.shop.request.review import ReviewModel
from shop_system_models.shop_api.shop.response.products import PaginationResponseModel
from shop_system_models.shop_api.shop.response.users import UserResponseModel


class ReviewResponseModel(ReviewModel):
    id: str
    user: UserResponseModel | None = None


class ReviewListResponseModel(BaseModel):
    reviews: list[ReviewResponseModel]
    page_info: PaginationResponseModel



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class ShopDetailsModel(BaseModel):
    shop_id: str
    shop_name: str
    friendly_name: str
    shop_language: str = "RU"
    shop_api_url: str = "https://marketplace-api-dev.tg-shops.com"
    contact_phone: str | None = None
    contact_email: str
    orders_chat_id: int
    orders_history_chat_id: Optional[int] = None
    search_enabled: Optional[bool] = False
    warehouse_accounting: bool = False
    bot_url: str | None = None
    bot_token: str
    payment_token: str | None = None
    placeholder: str
    order_process: str
    nocodb_project_id: str
    nocodb_categories_table: str
    nocodb_products_table: str
    nocodb_orders_table: str
    nocodb_status_table: str
    nocodb_bot_commands_table: str
    nocodb_options_table: Optional[str] = None
    nocodb_options_category_table: Optional[str] = None
    topic_chat_id: Optional[int] = None
    premium_expiration_date: Optional[datetime] = None
    web_app_url: str = ""



from pydantic import BaseModel

from shop_system_models.shop_api.processes.request import ProcessModel


class ProcessResponseModel(ProcessModel):
    id: str


class UpdateMessageResponseModel(BaseModel):
    text: str
    status: str
    order_number: str



from typing import Optional

from pydantic import BaseModel


class UpdateBotConfiguration(BaseModel):
    bot_token: str
    orders_chat_id: Optional[int] = None


class UpdateBotChatId(BaseModel):
    chat_id: Optional[int] = None



import enum

from pydantic import BaseModel, field_validator

from shop_system_models.shop_api.shop.request.products import (
    ProductModel,
    SubmitProductCategoriesModel,
)
from shop_system_models.shop_api.shop.response.extra_product_options import (
    ExtraOptionCategoriesResponseModel,
)


class BaseProductResponseModel(ProductModel, SubmitProductCategoriesModel):
    id: str


class ProductCheckoutModes(enum.Enum):
    PAYMENT = "payment"
    BOOKING = "booking"


class ProductResponseModel(BaseProductResponseModel):
    extra_option_choice_required: bool = False
    extra_option_categories: list[ExtraOptionCategoriesResponseModel] = []
    related_products: list["ProductResponseModel"] | None = []
    secret_urls: list[str] = []
    checkout_modes: list[ProductCheckoutModes] = []

    @field_validator("checkout_modes")
    def convert_status(cls, value: list[ProductCheckoutModes]):
        return [v.value for v in value]


class PartialPaginationResponseModel(BaseModel):
    total_rows: int
    is_first_page: bool
    is_last_page: bool


class PaginationResponseModel(PartialPaginationResponseModel):
    page: int
    page_size: int


class ProductListResponseModel(BaseModel):
    products: list[ProductResponseModel]
    page_info: PaginationResponseModel



from typing import Any, Dict, List, Optional

from pydantic import Field, validator

from shop_system_models.shop_api import MetaBaseModel


class ShopCreateRequest(MetaBaseModel):
    """Request model for shop creation."""

    name: str
    description: Optional[str] = None
    owner_id: str
    categories: List[str] = Field(default_factory=list)

    @validator("name")
    def validate_name(self, v):
        if not v.strip():
            raise ValueError("Shop name cannot be empty")
        return v


class ShopUpdateRequest(MetaBaseModel):
    """Request model for shop update."""

    shop_id: str
    name: Optional[str] = None
    description: Optional[str] = None
    categories: Optional[List[str]] = None
    settings: Optional[Dict[str, Any]] = None



from typing import List

from pydantic import BaseModel

from shop_system_models.shop_api.shop.request.orders import DeliveryTypeModel, OrderModel
from shop_system_models.shop_api.shop.response.products import PaginationResponseModel


class OrderResponseModel(OrderModel):
    """Response model for a single order."""

    id: str


class OrderListResponseModel(BaseModel):
    """Response model for a list of orders."""

    orders: List[OrderResponseModel]
    page_info: PaginationResponseModel


class DeliveryTypeResponseModel(DeliveryTypeModel):
    """Response model for a delivery type."""

    id: str



from enum import Enum

from pydantic import BaseModel

from shop_system_models.shop_api.shop.request.baskets import BasketModel, BasketModelV2
from shop_system_models.shop_api.shop.response.products import ProductResponseModel


class BasketResponseModel(BasketModel):
    id: str | None = None


class BasketResponseModelV2(BasketModelV2):
    id: str | None = None


class ProductsInBasket(ProductResponseModel):
    count_in_basket: int = 1


class UserBasketResponseModel(BasketResponseModel):
    products: list[ProductsInBasket] = []


class UserBasketResponseModelV2(BasketResponseModelV2):
    products: list[ProductsInBasket] = []


class ManageOrderAction(Enum):
    create = "create"
    delete = "delete"


class ManageBasketModel(BaseModel):
    message: str
    product_unique_id: str | None = None
    product: ProductResponseModel | None = None
